package springbootdemo;

import java.util.*;
import org.springframework.boot.env.EnumerableCompositePropertySource;
import org.springframework.core.env.*;

/**
 * Hilfsmethoden zu den Spring-Environment-Variablen und Properties in {@link ConfigurableEnvironment}.<br>
 * Ein ConfigurableEnvironment kann beispielsweise erzeugt werden per:<br>
 *    {@link ConfigurableApplicationContext#getEnvironment()},<br>
 *    {@link ConfigFileApplicationListener#postProcessEnvironment()},<br>
 *    {@link ApplicationEnvironmentPreparedEvent#getEnvironment()},<br>
 *    {@link EnvironmentPostProcessor#postProcessEnvironment()}<br>
 */
public class SpringEnvironmentUtil
{
   /**
    * Hinzufuegen einer einzelnen Property in einer neuen PropertySource.
    * Es darf im Spring-Environment noch keine PropertySource mit dem neuen PropertySource-Namen geben.
    * Die neue PropertySource wird vorrangig hinzugefuegt, andere Properties mit gleichem Key werden ueberdeckt.
    * @param  env          ConfigurableEnvironment, s.o. {@link SpringEnvironmentUtil}
    * @param  propSrcName  Name der neuen PropertySource, es darf noch keine PropertySource mit diesem Namen geben
    * @param  propKey      Key der Property
    * @param  propValue    Wert der Property
    */
   public static void addPropToSpringEnvironment( ConfigurableEnvironment env,
                                                  String propSrcName, String propKey, Object propValue )
   {
      Map<String,Object> props = new HashMap<String,Object>();
      props.put( propKey, propValue );
      addPropsToSpringEnvironment( env, propSrcName, props );
   }

   /**
    * Hinzufuegen mehrerer neuer Properties in einer neuen PropertySource.
    * Es darf im Spring-Environment noch keine PropertySource mit dem neuen PropertySource-Namen geben.
    * Die neue PropertySource wird vorrangig hinzugefuegt, andere Properties mit gleichem Key werden ueberdeckt.
    * @param  env          ConfigurableEnvironment, s.o. {@link SpringEnvironmentUtil}
    * @param  propSrcName  Name der neuen PropertySource, es darf noch keine PropertySource mit diesem Namen geben
    * @param  props        Hinzuzufuegende Properties
    */
   public static void addPropsToSpringEnvironment( ConfigurableEnvironment env,
                                                   String propSrcName, Map<String,Object> props )
   {
      PropertySource<?> propertySource = new MapPropertySource( propSrcName, props );
      env.getPropertySources().addFirst( propertySource );
   }

   /**
    * Ersetzen des Werts einer einzelnen Property in einer vorhandenen PropertySource.
    * Funktioniert nur, wenn es die PropertySource bereits gibt und diese eine Map oder EnumerableCompositePropertySource enthaelt.
    * @param  env          ConfigurableEnvironment, s.o. {@link SpringEnvironmentUtil}
    * @param  propSrcName  Name der PropertySource, z.B. "applicationConfigurationProperties" oder
    *                                                    "applicationConfig: [classpath:/application.properties]"
    * @param  propKey      Key der Property
    * @param  propNewValue Neuer Wert der Property
    * @return vorheriger Wert der Property, falls die Property geaendert werden konnte; sonst null
    */
   @SuppressWarnings("unchecked")
   public static Object replacePropInSpringEnvironment( ConfigurableEnvironment env,
                                                        String propSrcName, String propKey, String propNewValue )
   {
      MutablePropertySources mps = env.getPropertySources();
      PropertySource<?> ps = mps.get( propSrcName );
      if( ps != null ) {
         Object srcObj = ps.getSource();
         if( srcObj instanceof List<?> ) {
            for( Object obj1 : (List<?>) srcObj ) {
               if( obj1 instanceof EnumerableCompositePropertySource ) {
                  Collection<PropertySource<?>> coll = ((EnumerableCompositePropertySource) obj1).getSource();
                  if( coll instanceof Set ) {
                     for( Object obj2 : (Set<?>) coll ) {
                        if( obj2 instanceof PropertiesPropertySource ) {
                           srcObj = ((PropertiesPropertySource) obj2).getSource();
                        }
                     }
                  }
               }
            }
         }
         if( srcObj instanceof Map<?,?> ) {
            try {
               return ((Map<String,Object>) srcObj).put( propKey, propNewValue );
            } catch( ClassCastException e ) { /* ok */ }
         }
      }
      return null;
   }

   /**
    * Ermittlung aller Environment-Werte bzw. Properties in einer PropertySource.
    * @param  env          ConfigurableEnvironment, s.o. {@link SpringEnvironmentUtil}
    * @param  propSrcName  Name der PropertySource, z.B. "applicationConfigurationProperties" oder
    *                                                    "applicationConfig: [classpath:/application.properties]"
    */
   public static Map<String,Object> getMapFromSpringEnvironment( ConfigurableEnvironment env, String propSrcName )
   {
      Map<String,Object> mp = new TreeMap<String,Object>();
      for( Iterator<PropertySource<?>> itr = env.getPropertySources().iterator(); itr.hasNext(); ) {
         PropertySource<?> ps = itr.next();
         if( propSrcName != null && ps.getName() != null && !ps.getName().contains( propSrcName ) ) {
            continue;
         }
         if( ps instanceof EnumerablePropertySource ) {
            EnumerablePropertySource<?> eps = (EnumerablePropertySource<?>) ps;
            String[] pns = eps.getPropertyNames();
            for( String pn : pns ) {
               mp.put( pn, eps.getProperty( pn ) );
            }
         }
      }
      return mp;
   }
}
